#include "Delay.h"
#include <cstdlib>
#include <cmath>

Delay :: Delay(unsigned long maxDelayIn, double delayLengthIn)
{
	maxDelay = maxDelayIn;
	delayLengthInt = floor(delayLengthIn);
	delayLine = new SAMPLE[maxDelay];

	delta = delayLengthIn - delayLengthInt;	//fractional delay
	
	//Delta should be about 0.5 to 1.5 for flattest phase response
	if ( delta < 0.5 )
	{
		delayLengthInt = delayLengthInt - 1;
		delta += 1.0;	
	}

	eta = (1.0 - delta) / (1.0 + delta);	//calculate eta allpass filter coefficient
	
	//Initialize delayLine to hold all zeros
	clear();
	
	writeIndex = delayLengthInt;
	readIndex = 0;
}


Delay :: ~Delay()
{
	delete [] delayLine;
}


SAMPLE Delay :: tick(SAMPLE inSample)
{
	delayLine[writeIndex++] = inSample;
	//SAMPLE integerLengthOut = delayLine[readIndex++];
	
	//allpass filter
	lastOut = -eta*lastOut + eta*delayLine[readIndex] + allpassState;	//total delay line output
	allpassState = delayLine[readIndex++];	//update allpass filter state
	
	//Implement circular buffer
	if (writeIndex >= maxDelay-1) writeIndex = 0;
	if (readIndex >= maxDelay-1) readIndex = 0;
	
	return lastOut;
}


void Delay :: setDelayLength(double delayLengthIn)
{
	delayLengthInt = floor(delayLengthIn);
		
	delta = delayLengthIn - delayLengthInt;	//fractional delay

	//Delta should be about 0.5 to 1.5 for flattest phase response
	if ( delta < 0.5 )
	{
		delayLengthInt = delayLengthInt - 1;
		delta += 1.0;	
	}

	eta = (1.0 - delta) / (1.0 + delta);	//calculate eta allpass filter coefficient
	
	if ( writeIndex >= delayLengthInt ) readIndex = writeIndex - delayLengthInt;
	else readIndex = maxDelay - delayLengthInt + writeIndex;

}

void Delay :: clear()
{
	for (unsigned long i=0; i<maxDelay; i++)
	{
		delayLine[i] = 0.0;	
	}
	allpassState = 0.0;
	lastOut = 0.0;
}

void Delay ::	fillWithWhiteNoise(int velocity)
{
	for (unsigned long i=0; i<maxDelay; i++)
	{
		delayLine[i] = 2 * ((rand() / double(RAND_MAX)) - 0.5) * pow((velocity / 127.0),2);	
	}	
	allpassState = 2 * ((rand() / double(RAND_MAX)) - 0.5) * pow((velocity / 127.0),2);
}