#if !defined(__KSINSTRUMENT_H)
#define __KSINSTRUMENT_H

#include "KSString.h"
#include <list>

using namespace std;

//Karplus Strong string model instrument class
//Contains a piano's worth of KSStrings and keeps track of their activity.

//Note: this class has no error checking (for delay line lengths and such).

class KSInstrument
{
public:

	//Constructor
	KSInstrument(int numberOfStringsIn, int numberOfVoicesIn, double lowNoteFreqIn, double feedbackGainIn, double fsIn);
	
	//Destructor
	~KSInstrument();
	
	//Plucks the given string and marks it as active (being played) if we haven't reached the voice count limit specified during instantiation.
	void pluckString(int stringNum, int velocity);
	
	//Marks the given string as no longer active (no longer being played)
	void markInactive(int stringNum);
	
	//Gets the sum of the next output samples from all active strings and increments time one sample for those strings
	SAMPLE tick();
	
	//Utility fns
	void setFeedbackGain(double feedbackGainIn);
	double getFeedbackGain() { return feedbackGain; };
	void clear();
	
private:
	
	KSString** ksStringsCollection;
	
	//String parameters
	double lowNoteFreq;
	double feedbackGain;
	
	double fs;	//Sampling rate, used for frequency calculations
	
	int numberOfStrings;//Number of strings in the instrument
	int numberOfVoices; //Polyphony maximum
	
	list<int> activeStrings; //List which keeps track of active strings by their indices
	list<int>::iterator iter;

};

#endif
