#include "KSString.h"
#include <math.h>

//Constructor
KSString :: KSString(double lowestFreqIn, double frequencyIn, double fsIn, double feedbackGainIn)
{
	unsigned long maxDelay = ceil(1/lowestFreqIn * fsIn) + 1;	// 1 extra sample for safety (incase ceil doesn't round)
	double delayLength = 1/frequencyIn * fsIn - 0.5;	//Subtract 0.5 samples because our low-pass feedback damping filter has a half sample of linear phase delay
	delayLine = new Delay(maxDelay, delayLength);
	
	fs = fsIn;
	frequency = frequencyIn;
	feedbackGain = feedbackGainIn;
}

//Destructor
KSString :: ~KSString()
{
	delete delayLine;	
}

void KSString :: setFrequency(double frequencyIn)
{
	delayLine->setDelayLength( 1/frequencyIn * fs - 0.5 );	//Subtract 0.5 samples because our low-pass feedback damping filter has a half sample of linear phase delay
	frequency = frequencyIn;
}

//clears the contents of the delay line
void KSString :: clear()
{
	delayLine->clear();	
}

//Fills the delay line with white noise
void KSString :: pluck(int velocity)
{
	delayLine->fillWithWhiteNoise(velocity);	
}

//Gets the next output sample and increments time one sample. Implements a linear phase one zero lowpass filter in its feedback loop.
SAMPLE KSString :: tick()
{
	return delayLine->tick(feedbackGain * (delayLine->getNextOut() + delayLine->getLastOut())/2);
}